'use strict';

var _rimraf2 = require('rimraf');

var _rimraf3 = _interopRequireDefault(_rimraf2);

var _parseGithubUrl = require('parse-github-url');

var _parseGithubUrl2 = _interopRequireDefault(_parseGithubUrl);

var _gunzipMaybe = require('gunzip-maybe');

var _gunzipMaybe2 = _interopRequireDefault(_gunzipMaybe);

var _meow = require('meow');

var _meow2 = _interopRequireDefault(_meow);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _package = require('../../package.json');

var _package2 = _interopRequireDefault(_package);

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

var _request = require('request');

var _request2 = _interopRequireDefault(_request);

var _sander = require('sander');

var _sander2 = _interopRequireDefault(_sander);

var _shellEscape = require('shell-escape');

var _shellEscape2 = _interopRequireDefault(_shellEscape);

var _crossSpawn = require('cross-spawn');

var _crossSpawn2 = _interopRequireDefault(_crossSpawn);

var _figures = require('figures');

var _tarFs = require('tar-fs');

var _tarFs2 = _interopRequireDefault(_tarFs);

var _updateNotifier = require('update-notifier');

var _updateNotifier2 = _interopRequireDefault(_updateNotifier);

var _child_process = require('child_process');

var _chalk = require('chalk');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const rimraf = (0, _bluebird.promisify)(_rimraf3.default);

const prompt = (0, _chalk.grey)('>');

const helpText = `
    1. Downloads a snapshot of the specified repo to your current
       working directory
    2. Does ${ (0, _chalk.cyan)('git init') }, ${ (0, _chalk.cyan)('git add .') } and ${ (0, _chalk.cyan)('git commit') }
    3. If there's a package.json, runs ${ (0, _chalk.cyan)('npm install') }

    ${ (0, _chalk.grey)('──────────────────────────────────────────────────────────────────────') }

    Usage
      ${ prompt } startfrom ${ (0, _chalk.cyan)('user') }/${ (0, _chalk.cyan)('repo') }${ (0, _chalk.grey)('[') }#${ (0, _chalk.cyan)('ref') }${ (0, _chalk.grey)(']') } ${ (0, _chalk.grey)('[') }${ (0, _chalk.cyan)('subdirectory') }${ (0, _chalk.grey)(']') }

    Examples:
      ${ prompt } startfrom google/web-starter-kit
      ${ prompt } startfrom google/web-starter-kit#v0.5.4
      ${ prompt } startfrom h5bp/html5-boilerplate#v5.0.0 dist
      ${ prompt } startfrom https://github.com/mxstbr/react-boilerplate
`;

const cli = (0, _meow2.default)(helpText);

// if no CLI args, do the same as `startfrom --help`
if (!cli.input[0]) {
  console.log(helpText);
  process.exit(0);
}

const dir = process.cwd();

var _ghParse = (0, _parseGithubUrl2.default)(cli.input[0]);

const owner = _ghParse.owner;
const name = _ghParse.name;
const ref = _ghParse.branch;

const subdirectory = cli.input[1] || '.';

// check for updates
const notifier = (0, _updateNotifier2.default)({ pkg: _package2.default });
notifier.notify();

function run() {
  for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
    args[_key] = arguments[_key];
  }

  return new _bluebird2.default((resolve, reject) => {
    console.log((0, _chalk.grey)(' > ') + (0, _shellEscape2.default)(args));

    const child = (0, _crossSpawn2.default)(args.shift(), args, {
      stdio: 'inherit',
      cwd: dir
    });

    child.on('error', reject);

    child.on('close', code => {
      if (code !== 0) reject(new Error('Command failed'));else resolve();
    });
  });
}

// begin the async sequence
(0, _bluebird.coroutine)(function* () {
  if (!owner || !name) throw new Error(`Invalid: ${ cli.input[0] }`);

  console.log('');

  const files = yield _sander2.default.readdir(dir);

  // the directory must be empty (except for .git and .DS_Store), otherwise we exit
  if (files.filter(file => file !== '.DS_Store' && file !== '.git').length) {
    console.log((0, _chalk.red)('This directory is not empty!'));
    console.log('\nPlease ' + (0, _chalk.cyan)('cd') + ' into an empty directory and try again.');
    process.exit(1);
  }

  // if the directory is already git-managed, that is ok providing there are no
  // uncommitted changes (this covers the scenario where user has deleted all
  // existing files from an existing project in order to do a 'fresh start'
  // with the starter kit while retaining history)
  const hasGit = files.indexOf('.git') > -1;
  if (hasGit) {
    say(`This is alreay a git-managed directory; verifying working directory is clean...`);

    try {
      (0, _child_process.execSync)('git diff --exit-code');
    } catch (error) {
      console.log((0, _chalk.red)(`\nWorking directory is not clean.`));
      console.log('Please commit your changes then try running this script again.');
      process.exit(1);
    }
    doTick();
  }

  // download whole repo tarball into ./__startfrom_tmp
  yield new _bluebird2.default((resolve, reject) => {
    const tarballURL = `https://github.com/${ owner }/${ name }/archive/${ ref }.tar.gz`;
    say('Downloading and extracting ' + tarballURL);

    (0, _request2.default)(tarballURL).pipe((0, _gunzipMaybe2.default)()).pipe(_tarFs2.default.extract(dir, {
      map(header) {
        header.name = '__startfrom_tmp/' + header.name.substring(header.name.indexOf('/') + 1);
      }
    }).on('finish', resolve)).on('error', reject);
  });

  // copy over all the contents of the requested subdirectory
  yield _bluebird2.default.map(_sander2.default.readdir(dir, '__startfrom_tmp', subdirectory), file => _sander2.default.rename(dir, '__startfrom_tmp', subdirectory, file).to(dir, file));

  // then rimraf the temp dir
  yield rimraf(_path2.default.resolve(dir, '__startfrom_tmp'));

  // commit everything...
  say(`Committing initial state...`);
  if (!hasGit) yield run('git', 'init');
  yield run('git', 'add', '.');
  yield run('git', 'commit', '-m', 'startfrom ' + (0, _shellEscape2.default)(process.argv.slice(2)));
  doTick();

  // run npm install if there's a package.json
  if (yield _sander2.default.exists(dir, 'package.json')) {
    say(`Running npm install for you (this might take a while)...`);
    yield run('npm', 'install');
    doTick();
  }

  console.log('\n\n' + _chalk.bgGreen.black(` ${ _figures.tick } ALL DONE! `));
})();

function say(message) {
  console.log((0, _chalk.yellow)(`\n\n${ message }`));
}

function doTick() {
  console.log((0, _chalk.green)(_figures.tick));
}
//# sourceMappingURL=index.js.map